
function love.load ()
	require "skiplist"
	G = love.graphics
	G.setBackgroundColor(25, 25, 25)

	pick_results = G.newCanvas(1, 1)
	id_canvas    = G.newCanvas()	
	paint_ids    = G.newPixelEffect "paint_ids.glsl"
	pick_object  = G.newPixelEffect "pick_object.glsl"

	pick_object:send("id_canvas", id_canvas)
	id_canvas:setFilter("nearest", "nearest") -- necessary so ids are not interpolated

	objects = load_objects()
	function objects:draw ()
		for k, obj in objects:iterate() do
			draw(k, obj.image, obj.x, obj.y, 0, 1, 1, obj.ox, obj.oy)
		end
	end
end

local random = math.random
function load_objects ()
	max_id = 0
	local objects = skiplist.new()
	local files	= love.filesystem.enumerate "images"
	for _, file in ipairs(files) do
		local img = G.newImage("images/"..file)
		local obj = {
			image = img;
		   ox = img:getWidth()/2;
		   oy = img:getHeight()/2;
		   x = random(G.getWidth()/2) + G.getWidth()/4;
		   y = random(G.getHeight()/2) + G.getHeight()/4;
		}
		max_id = max_id + 1
		objects:insert(max_id, obj)
	end
	return objects
end

function draw (id, ...)
	G.setCanvas(id_canvas)
	G.setPixelEffect(paint_ids)
	paint_ids:send("id", id)
	G.draw(...)
	G.setCanvas()
	G.setPixelEffect()
	G.draw(...)
end

function get_id_at (x, y)
	G.setPixelEffect(pick_object)
	pick_object:send("pick_coords", {x/G.getWidth(), 1-y/G.getHeight()})
	G.setCanvas(pick_results)
	G.rectangle("fill", 0, 0, 1, 1)
	local data = pick_results:getImageData()
	G.setCanvas()
	G.setPixelEffect()
	return (data:getPixel(0, 0))
end

function pick_up_obj (id)
	if held_object ~= nil then return end
	obj = objects:search(id)
	if obj == nil then return end
	objects:delete(id)
	held_object = obj
end

function put_down_object (x, y)
	if not held_object then return end
	held_object.x = x
	held_object.y = y
	max_id = max_id + 1
	objects:insert(max_id, held_object)
	held_object = nil
end

function love.mousepressed (x, y)
	pick_up_obj(get_id_at(x, y))
end

function love.mousereleased (x, y)
	put_down_object(x, y)
end

function love.draw ()
	id_canvas:clear()
	objects:draw()
	if held_object then
		local ho = held_object
		local x, y = love.mouse.getPosition()
		G.draw(ho.image, x, y, 0, 1, 1, ho.ox, ho.oy)
	end
end

