
--[[
Usage:
require "skiplist"
list = skiplist.new()
list:insert(5, "foo bar baz")
list:search(5) --> "foo bar baz"
list:delete(5)
list:search(5) --> nil

-- this prints all key value pairs in order:
for k, v in list:iterate() do
   print(k, v)
end
--]]

-- if this is nowhere near correct, change it.
local approximate_maximum_number_of_elements = 1024
local max_level = math.ceil(math.log(approximate_maximum_number_of_elements)/math.log(2))

local nil_node = {}
local mt = {}

skiplist = {}

local function comp (a, b)
   if a == nil then
      return false
   elseif b == nil then
      return false
   else
      return a < b
   end
end

function skiplist.new ()
   local list = {
      level = 1,
      header = {
         forward = setmetatable({}, {__index = function () return nil_node end})
      }
   }
   return setmetatable(list, mt)
end

local function search (list, search_key)
   local x = list.header
   for i = list.level, 1, -1 do
      while comp(x.forward[i].key, search_key) do
         x = x.forward[i]
      end
   end
   x = x.forward[1]
   if x.key == search_key then
      return x.value
   else
      return nil
   end
end

local random = math.random
local function random_level ()
   local level = 1
   while random() < 0.5 and level < max_level do
      level = level + 1
   end
   return level
end

local function insert (list, search_key, new_value)
   local update = {}
   local x = list.header
   for i = list.level, 1, -1 do
      while comp(x.forward[i].key, search_key) do
         x = x.forward[i]
      end
      update[i] = x
   end
   x = x.forward[1]
   if x.key == search_key then
      x.value = new_value
   else
      local level = random_level()
      if level > list.level then
         for i = list.level+1, level do
            update[i] = list.header
         end
         list.level = level
      end
      x = {key=search_key, value=new_value, forward={}}
      for i = 1, level do
         x.forward[i] = update[i].forward[i]
         update[i].forward[i] = x
      end
   end
end

local function delete (list, search_key)
   local update = {}
   x = list.header
   for i = list.level, 1, -1 do
      while comp(x.forward[i].key, search_key) do
         x = x.forward[i]
      end
      update[i] = x
   end
   x = x.forward[1]
   if x.key == search_key then
      for i = 1, list.level do
         if update[i].forward[i] ~= x then
            break
         end
         update[i].forward[i] = x.forward[i]
      end
      while list.level > 1 and list.header.forward[list.level] == nil do
         list.level = list.level - 1
      end
   end
end

local function iterate (list)
   return coroutine.wrap(function ()
      local x = list.header
      while x.forward[1] ~= nil_node do
         x = x.forward[1]
         coroutine.yield(x.key, x.value)
      end
   end)
end

mt.__index = {
   search=search,
   insert=insert,
   delete=delete,
   iterate=iterate,
}

function skiplist.test ()
   local list = skiplist.new()
   local t = {}
   for _ = 1, 1000 do
      local k, v = random(100000), random(100000)
      list:insert(k, v)
      t[k] = v
   end
   for k, v in pairs(t) do
      assert(list:search(k) == v)
   end
   list = skiplist.new()
   for _ = 1, 1000 do
      local x = random(10000)
      list:insert(x, x)
   end
   local prev
   for k, v in list:iterate() do
      if prev then
         assert(prev < v)
      end
      prev = v
   end
   print "test successful"
   return true
end
